function [K,f] = constrain(Kg,C,L)
% constrain  Apply constraints by eliminating equations from global matrices
%            Both the stiffness matrix and the load vector must be trimmed
%
% Synopsis:  [K,f] = constrain(Kg,C,L)
%
% Input:   Kg = global stiffness matrix without any constraints
%          C  = matrix of contraint data.  Column 1 is global node number (unused)
%               Column 2 is global DOF.  Column 3 is value of contraint for global DOF
%          L  = matrix of load data.  Column 1 is global node number (unused)
%               Column 2 is global DOF.  Column 3 is value of load for global DOF
%
% Output:  K = stiffness matrix, modified so that solution will impose constraints
%          f = load vector with applied loads and modified so that solution with
%              meet the constraints

% Copyright (c), 1998, Gerald Recktenwald, All rights reserved
% This m-file may be freely distributed, but it may not be sold.
% Version 1.0  22 Feb 1998,  gerry@me.pdx.edu

[mdof,ndof] = size(Kg);  %  mdof should equal ndof = number of global degrees of freedom
if mdof~=ndof
  error('Global stiffness matrix is not square');
end

% --- Create load vector.  First apply true loads. Later modify f to enforce constraints
f = zeros(ndof,1);       %  Initialize rhs (load) vector
f( L(:,2) ) = L(:,3);    %  Store prescribed loads:
                         %     L(:,2) = indices of global DOFs
                         %     L(:,3) = values of loads for global DOFs

% --- Create constrained stiffness matrix
K = Kg;         %  Copy entire global stiffness matrix as starting point
nc = C(:,2);    %  global DOF indices of contrained nodes
cv = C(:,3);    %  constraint values

for i=1:length(nc)
   ic = nc(i);            %  Index of global DOF to be constrained
   f = f - K(:,ic)*cv(i); %  Add influence of constraint to rhs of other equations
                          %    K(:,ic) is mostly zeros.  Non-zero elements are in equations
                          %    of global DOF that are coupled to DOF that is constrained (ic)
   K(ic,:) = 0;           %  Decouple constrained nodes from dependence on other nodes.
   K(:,ic) = 0;           %  Decouple all other equations from the constrained node 
   K(ic,ic) = 1;          %  Reset diagonal coefficient of constrained node
   f(ic) = cv(i);         %  Guarantees that solution is equal to value in cv(i),
end

% ALTERNATE FORMULATION:
%   Extract d = original diagonal element of constrained rows before row is zeroed.
%   Then reset the diagonal elements and set correpsonding element of the rhs vector
%   to cv(i)/d(i).  The row then reads  d(i)*x(i) = cv(i)/d(i), which has the
%   solution: x(i) = cv(i)
% 
% for i=1:length(nc)
%    ic = nc(i);            %  Index of global DOF to be constrained
%    f = f - K(:,ic)*cv(i); %  Add influence of constraint to rhs of other equations
%                           %    K(:,ic) is mostly zeros.  Non-zero elements are in equations
%                           %    of global DOF that are coupled to DOF that is constrained (ic)
%    d = K(ic,ic);          %  Save value of diagonal element of constrained node
%    K(ic,:) = 0;           %  Decouple constrained nodes from dependence on other nodes.
%    K(:,ic) = 0;           %  Decouple all other equations from the constrained node 
%    K(ic,ic) = 1;          %  Reset diagonal coefficient of constrained node
%    f(ic) = cv(i)/(d+eps); %  Guarantees that solution is equal to value in cv(i),
% end                       %    extra eps prevents NaN when dkc(i) = 0
