function drawMesh2d(XY,E,meshColor,drawEl,labelEl)
% drawMesh  Draw the FE mesh for a 2D truss structure
%
% Synopsis:  drawMesh2d(XY,E,meshColor,drawEl,labelEl)
%
% Input:     XY = nn by 2 matrix of global node positions, where nn is the
%                 number of nodes.  XY(i,1) is the x position of node i
%                 XY(i,2) is the y position.
%            E  = ne by 3 matrix of element definitions, where ne is the
%                 number of elements.  E(i,1) and E(i,2) are the global
%                 node numbers that define element i.  E(i,3) is an integer
%                 the type of material -- see M matrix, below
%            meshColor = (string) character indicating color used to draw
%                        nodes and elements.  Valid colors are those defined
%                        for the MATLAB plot command, i.e.
%
%                            symbol   color
%                               y     yellow
%                               m     magenta
%                               c     cyan 
%                               r     red   
%                               g     green
%                               b     blue
%                               w     white
%                               k     black
%
%                        Dark colors such as 'r', 'b', and 'k' are recommended
%                        Default:  meshColor = 'b' (blue)
%            drawEl = flag used to control whether elements are drawn
%                      Default: drawEl = 1 (true).  Use drawEl = 0 to draw nodes only
%            labelEl = flag used to control whether elements are labelled with element
%                      numbers.  Default: drawEl = 1 (true).  Use labelEl = 0 to
%                      suppress labeling of elements.
%
% Output:    plot of nodes and elements

% Copyright (c), 1998, Gerald Recktenwald, All rights reserved
% This m-file may be freely distributed, but it may not be sold.
% Version 1.0  22 Feb 1998,  gerry@me.pdx.edu

if nargin<3,  meshColor = 'b';  end
if nargin<4,  drawEl = 1;       end
if nargin<5,  labelEl = 1;      end

% --- Extract coordinates to separate vectors for ease of reading code
x = XY(:,1);  y = XY(:,2);

% --- draw the nodes
nodeSymbol = [meshColor,'o'];
plot(x,y,nodeSymbol);

% --- expand the plot frame so mesh is contained with small borders
f = 0.2;                     %  100*f = percent to expand the plot frame
xmin = min(x);  xmax = max(x);  ymin = min(y);  ymax = max(y);
border = 0.5*f*max( [xmax-xmin ymax-ymin ] );
xmins = xmin - border;
xmaxs = xmax + border;
ymins = ymin - border;
ymaxs = ymax + border;
axis('equal');                            %  show true lengths in x and y
axis([ xmins  xmaxs  ymins  ymaxs ]);


% --- label the elements:  requires some heuristics to locate element numbers
[ne,ncols] = size(E);
i1 = E(:,1);   i2 = E(:,2);   %  global DOFs for nodes in each element
hold_state = ishold;          %  preserve current hold state

if ~hold_state,  hold on;  end
elementSymbol = [meshColor,'-'];
if drawEl
  for n=1:ne
    plot([x(i1(n)) x(i2(n))],[y(i1(n)) y(i2(n))],elementSymbol);
  end
end

if labelEl
  xmax = max(abs(x(i1)-x(i2)));    %  largest x-length of all elements
  ymax = max(abs(y(i1)-y(i2)));    %  largest y-length of all elements
  xmid = 0.5*(x(i1)+x(i2));        %  x-direction midpoint of element
  ymid = 0.5*(y(i1)+y(i2));        %  x-direction midpoint of element
  hvtol = xmax/1e6;                    %  tolerance used to detect horizontal
  for n=1:ne
    % --- find good location for element labels
    if abs(xmid(n)-x(i1(n)))<hvtol         %  vertical element
      xtext = xmid(n)+0.04*xmax;
      ytext = ymid(n);
    elseif abs(ymid(n)-y(i1(n)))<hvtol     %  horizontal element
      xtext = xmid(n);
      ytext = ymid(n)-0.08*ymax;
    elseif (y(i2(n))-y(i1(n)))/(x(i2(n))-x(i1(n)))>0 %  element slopes up
      xtext = xmid(n)+0.05*xmax;
      ytext = ymid(n)-0.05*ymax;
    else                                     %  element slopes down
      xtext = xmid(n)-0.1*xmax;
      ytext = ymid(n)+0.05*ymax;
    end
    text(xtext,ytext,num2str(n));            %  write the element number
  end
end

if ~hold_state,  hold off;  end

