function [Kcout,fcout,Kgout] = frame2d(fin,verbose)
% frame2d  2D truss finite element program
%
% Synopsis:   K = frame2d               K = frame2d(fin)
%             [K,f] = frame2d           [K,f] = frame2d(fin)
%             [K,f,Ku] = frame2d        [K,f,Ku] = frame2d(fin)
%
% Input:      fin = (optional) name of file containing mesh definition
%                   Default:  longFrame.msh
%
% Output:     K  = (optional) constrained stiffness matrix of the frame
%             f  = (optional) load vector for the frame
%             Ku = (optional) unconstrained load vector for the frame
%
% Note:  if ANY output parameters are specified, the solution for the
%        displacements is not obtained

% Copyright (c), 1998, Gerald Recktenwald, All rights reserved
% This m-file may be freely distributed, but it may not be sold.
% Version 1.01  7 July 1998,  gerry@me.pdx.edu

if nargin<1,  fin = 'longFrame.msh';   end
if nargin<2,  verbose = 0;             end

fprintf('\nParsing input file...');
[XY,E,C,L,M,desc] = parseMesh(fin,verbose);
fprintf(' done\n\n');

K = stiffness(XY,E,M);          %  Unconstrained stiffness matrix
[Kc,fc] = constrain(K,C,L);     %  Apply loads and constraints

% --- Don't solve for displacements, user has requested stiffness matrices
if nargout==3
  Kcout=Kc;  fcout=fc;  Kgout=K;  return; 
elseif nargout==2
  Kcout=Kc;  fcout=fc;  return; 
elseif nargout==1
  Kcout=Kc;  return;
end

% --- Proceed with solution
q = Kc\fc;                         %  Solve for displacements of all DOF
r = K*q - fc;                      %  Reaction forces
XYd = displace(XY,q);              %  Position of nodes after displacement
[s,dl,dlr] = stress(XY,XYd,E,M);   %  Stress and length change of each element

% --- Draw orginal and deformed mesh
figMesh = figure;
drawMesh2D(XY,E,'b');          %  Original mesh
title(desc)
hold on

sf = 0.02*max(max(XY))/max(q);  %  Scale factor for exageration of displacements
XYs = displace(XY,q,sf);
drawMesh2D(XYs,E,'r',1,0)
hold off
 
% --- Print results
[nn,ndim] = size(XY);
fprintf('\n  DOF      x-disp        y-disp        x-react       y-react\n');
for i=1:nn
  ng = 2*(i-1);
  fprintf('%4d  %12.6f  %12.6f  %12.1f  %12.1f\n',i,q(ng+1),q(ng+2),r(ng+1),r(ng+2));
end

fprintf('\n                       Absolute\n');
fprintf('\nElement    stress    Length Change\n');
for i=1:length(s)
  fprintf('%4d  %12.1f     %8.4f\n',i,s(i),dl(i));
end

checkBalance(L,r);


% ====================================================================
function Xd = displace(X,q,sf)
% displace  Compute displaced node positions, with possible exageration
%
% Synopsis:  Xd = scaleDisp(X,q)
%            Xd = scaleDisp(X,q,sf)
%
% Input:     X = nn by ndim matrix of nodal positions, where nn = number
%                of nodes, ndim = number of dimensions, 2 or 3
%            q = nn*ndim vector of nodal displacements
%            sf = (optional) scale factor to exagerate the displacements
%                     Default:  sf = 1

if nargin<3,  sf = 1;  end

[nn,ndim] = size(X);
if ndim>2
  error('cannot handle 3D mesh yet');
end

nq = length(q);
Xd = X;                                   %  Extract nodal displacements
Xd(:,1) = Xd(:,1) + sf*q(1:2:nq-1);       %    x displacements are DOF 1,3,5,...
Xd(:,2) = Xd(:,2) + sf*q(2:2:nq);         %    y displacements are DOF 2,4,6,...


% ====================================================================
function checkBalance(L,r)
% checkBalance  Compute sum of loads and reaction forces in each direction
%
% Synopsis:  checkBalance(L,r)
%
% Input:   L  = matrix of load data.  Column 1 is global node number (unused)
%               Column 2 is global DOF.  Column 3 is value of load for global DOF
%          r = vector of reactions obtained by solution
%
% Output:    Printed table of force balances in x and y directions.

ix = find(rem(L(:,2),2)==1);  %  Column indices of elements in Load matrix that
                              %  that correspond to applied loads in x-direction
if isempty(ix)
   xload = 0;
else
   xload = sum(L(ix,3));
end
iy = find(rem(L(:,2),2)==0);  %  Column indices of elements in Load matrix that
                              %  that correspond to applied loads in y-direction
if isempty(iy)
   yload = 0;
else
   yload = sum(L(iy,3));
end

nr = length(r);
xreact = sum(r(1:2:nr-1));
yreact = sum(r(2:2:nr));

fprintf('\n\tForce balance on truss:\n');
fprintf('            x-direction    y-direction\n');
fprintf('loads     %12.2f   %12.2f\n',xload,yload);
fprintf('reactions %12.2f   %12.2f\n',xreact,yreact);
fprintf('            --------------------------\n');
fprintf('balance   %12.2f   %12.2f\n',xload+xreact,yload+yreact);
