function [XY,E,C,L,M,dstr] = parseMesh(fin,verbose)
% parseMesh  Reads 2D node and element definitions from text file
%
% Synopsis:  [XY,E,C,L,M] = parseMesh(fin,verbose)
%
% Input:     fin = (string) name of input file
%            verbose = (optional) flag controlling whether the input text
%                      file is printed to the screen.  Default:  verbose = 0
%
% Output:    XY = nn by 2 matrix of global node positions, where nn is the
%                 number of nodes.  XY(i,1) is the x position of node i
%                 XY(i,2) is the y position.
%            E  = ne by 3 matrix of element definitions, where ne is the
%                 number of elements.  E(i,1) and E(i,2) are the global
%                 node numbers that define element i.  E(i,3) is an integer
%                 the type of material -- see M matrix, below
%            C  = nc by 3 matrix of nodal constraints, where nc is total
%                 number of constraints.  C(i,1) is the global node number
%                 at which constraint i is applied.  C(i,2) is the global
%                 degree of freedom which is constrained.  (Note that C(i,1)
%                 and C(i,2) supply redundant info, but are retained for now.)
%                 C(i,3) is the value of the constraint (often zero).
%            L  = nl by 3 matrix of nodal loads, where nl is total
%                 number of constraints.  L(i,1) is the global node number
%                 at which load i is applied.  L(i,2) is the global
%                 degree of freedom which is loaded.  (Note that L(i,1)
%                 and Li,2) supply redundant info, but are retained for now.)
%                 L(i,3) is the value of the load.
%            M  = nm by 4 matrix of material definitions, where nm is the
%                 total number of materials.  M(i,1) is an integer material
%                 ID number.  Usually, M(i,1) = i, but this is not necessary.
%                 The ID number is used in the element definition matrix,
%                 see E(i,3), above.  M(i,2) is the Young's modulus for the
%                 material.  M(i,3) is the cross-sectional area of the truss
%                 element.  M(i,4) is the density (mass/vol) of the material.

% Copyright (c), 1998, Gerald Recktenwald, All rights reserved
% This m-file may be freely distributed, but it may not be sold.
% Version 1.01  7 July 1998,  gerry@me.pdx.edu

fp = fopen(fin,'r');
if fp<0,  error(sprintf('Could not open %s',fin));  end

% --- Read entire file into a string matrix.  This facilitates
%     backstepping in the file, which is necessary for current
%     way that nodelist and ellist are read
nextLine = 0;
inbuf = fgetl(fp);
if ~isstr(inbuf)
  error(sprintf('mesh file named %s could not be read',fname));
end

% --- Read contents of entire text file into a string buffer.
%     This makes it easier to move forward and backward in the file
%     without having to manage low level I/O.
while nextLine~=-1
  nextLine = fgetl(fp);
  if nextLine~=-1
     inbuf = str2mat(inbuf,nextLine);
  end
end

if verbose, disp(inbuf);  end

% --- Begin parsing the input file
[maxLines,ncol] = size(inbuf);

% --- Initialize return variables to null in case some are not defined
%     by data in the input file.  Undefined variables are not necessarily
%     a problem, depending on how the input file is used.
XY = [];  E = [];  C = [];  L = [];  M = [];  dstr = '';

nLine=0;
while nLine<maxLines
  nLine = nLine + 1;
  nextLine = inbuf(nLine,:);   %  get next row of input buffer

  if strcmp(nextLine(1),'%')
     ;   %  do nothing, line is a comment
  elseif strcmp(nextLine(1),'*')
     if strcmp(nextLine(2:5),'desc')
        nLine = nLine + 1;
        dstr = inbuf(nLine,:);        %  description string must be one line long
     elseif strcmp(nextLine(2:5),'node')
        [XY,nLine] = readNodes(inbuf,nLine,maxLines);
     elseif strcmp(nextLine(2:3),'el')
        [E,nLine] = readElements(inbuf,nLine,maxLines);
     elseif strcmp(nextLine(2:4),'con')
        [C,nLine] = readConstraints(inbuf,nLine,maxLines);
     elseif strcmp(nextLine(2:5),'load')
        [L,nLine] = readConstraints(inbuf,nLine,maxLines);
     elseif strcmp(nextLine(2:4),'mat')
        [M,nLine] = readMaterials(inbuf,nLine,maxLines);
     else
        fprintf('nextLine = %s',nextLine)
     end
  else
    fprintf('Error in parsing line %d of mesh definition in %s\n',nLine,fname);
    fprintf('Current line = \n\t%s\ndoes not contain valid command\n',nextLine);
    error('I give up');
  end
end


% ==================================
function [XY,nLine] = readNodes(inbuf,nLine,maxLines)
% readNodes  Read list of node definitions
%
% Synopsis:  [XY,nLine] = readNodes(inbuf,nLine)
%
% Input:     inbuf = string matrix containing input lines
%            nLine = index of current line in inbuf
%            maxLines = total number of lines in inbuf
%
% Output:    XY = n by 2 matrix of global node positions, where n is the
%                 number of nodes.  XY(i,1) is the x position of node i
%                 XY(i,2) is the y position.
%            nLine = index of next line to be read in inbuf

while nLine<maxLines & ~( strcmp(inbuf(nLine+1,1),'*') | strcmp(inbuf(nLine+1,1),'%') )
  nLine = nLine + 1;
  [nn,remain]    = getNumber(inbuf(nLine,:),inbuf(nLine,:));
  [x(nn),remain] = getNumber(remain,inbuf(nLine,:));
  [y(nn),remain] = getNumber(remain,inbuf(nLine,:));
% 
%   theLine = inbuf(nLine,:);
%   [token,remain] = strtok(theLine);
%   nodeNumber = str2num(token);
%   %  n = n+1;  %  ignore value of user's number??, make sure node numbers are in sequence
%   [token,remain] = strtok(remain);
%   x(nodeNumber) = str2num(token);
%   [token,remain] = strtok(remain);
%   y(nodeNumber) = str2num(token);
end

XY = [x(:) y(:)];

% ==================================
function [E,nLine] = readElements(inbuf,nLine,maxLines)
% readElements  Read list of element definitions
%
% Synopsis:  [E,nLine] = readElements(inbuf,nLine,maxLines)
%
% Input:     inbuf = string matrix containing input lines
%            nLine = index of current line in inbuf
%            maxLines = total number of lines in inbuf
%
% Output:    E     = n by 3 matrix of element definitions,
%                    where n = number of nodes.  Columns 1 and 2 are
%                    global node numbers that define the element. Third
%                    column is integer indicating the type of material
%            nLine = index of next line to be read in inbuf

while nLine<maxLines & ~( strcmp(inbuf(nLine+1,1),'*') | strcmp(inbuf(nLine+1,1),'%') )
  nLine = nLine + 1;
  [ne,remain]      = getNumber(inbuf(nLine,:),inbuf(nLine,:));
  [i1(ne),remain]  = getNumber(remain,inbuf(nLine,:));
  [i2(ne),remain]  = getNumber(remain,inbuf(nLine,:));
  [mat(ne),remain] = getNumber(remain,inbuf(nLine,:));
end

E = [i1(:) i2(:) mat(:)];

% ==================================
function [C,nLine] = readConstraints(inbuf,nLine,maxLines)
% readConstraints  Read list of constraints or loads, format is same for both
%
% Synopsis:  [C,nLine] = readConstraints(inbuf,nLine,maxLines)
%
% Input:     inbuf = string matrix containing input lines
%            nLine = index of current line in inbuf
%            maxLines = total number of lines in inbuf
%
% Output:    C  = n by 3 matrix of nodal constraints (or loads), where nc is total
%                 number of constraints (loads).  C(i,1) is the global node number
%                 at which constraint (load) i is applied.  C(i,2) is the global
%                 degree of freedom which is constrained.  (Note that C(i,1)
%                 and C(i,2) supply redundant info, but are retained for now.
%            nLine = index of next line to be read in inbuf

n = 0;
while nLine<maxLines & ~( strcmp(inbuf(nLine+1,1),'*') | strcmp(inbuf(nLine+1,1),'%') )
  nLine = nLine + 1;
  n = n + 1;
  [nn(n),remain]    = getNumber(inbuf(nLine,:),inbuf(nLine,:));
  [dof(n),remain]   = getNumber(remain,inbuf(nLine,:));
  [value(n),remain] = getNumber(remain,inbuf(nLine,:));
end

nc = 2*(nn-1) + dof;    %  global_DOF = 2*(node_num - 1) + local_DOF

C = [nn(:) nc(:) value(:)];

% ==================================
function [M,nLine] = readMaterials(inbuf,nLine,maxLines)
% readConstraints  Read list of materials
%
% Synopsis:  [M,nLine] = readConstraints(inbuf,nLine,maxLines)
%
% Input:     inbuf = string matrix containing input lines
%            nLine = index of current line in inbuf
%            maxLines = total number of lines in inbuf
%
% Output:    M     = n by 4 matrix of material definitions, where n is the
%                    total number of materials.  M(i,1) is an integer
%                    material ID number.  Usually, M(i,1) = i, but this is not
%                    necessary.  The ID number is used in the element definition
%                    matrix, see E(i,3), above.  M(i,2) is the Young's modulus
%                    for the material.  M(i,3) is the cross-sectional
%                    area of the truss element.  M(i,4) is the density
%                    (mass/vol) of the material.
%            nLine = index of next line to be read in inbuf

nm = 0;
while nLine<maxLines & ~( strcmp(inbuf(nLine+1,1),'*') | strcmp(inbuf(nLine+1,1),'%') )
  nLine = nLine + 1;
  nm = nm + 1;
  [id(nm),remain]   = getNumber(inbuf(nLine,:),inbuf(nLine,:));
  [e(nm),remain]    = getNumber(remain,inbuf(nLine,:));
  [area(nm),remain] = getNumber(remain,inbuf(nLine,:));
  [den(nm),remain]  = getNumber(remain,inbuf(nLine,:));
end

M = [id(:) e(:) area(:) den(:)];


% ==================================
function [x,remain] = getNumber(theString,theLine)
% getNumber  Convert the next token in input buffer to a number
%
% Synopsis:  [x,remain] = getNumber(theString,theLine)
%
% Input:     theString = segment of input line in which token parsing begins
%            theLine = entire input line;  used for error message only
%
% Output:    x = numerical value of the first token in 'theString'
%                separated by whitespace.  If x == [] the conversion failed
%                and an error message is printed.
%            remain = sub-string of 'theLine' immediately following the
%                     successful conversion.  remain is likely to be
%                     used in the next call to getNumber()

[token,remain] = strtok(theString);
x = str2num(token);
if isempty(x)
  fprintf('\n\tError in getNumber\n');
  fprintf('\tWhile reading line ''%s''\n',theLine);
  fprintf('\ttoken = %s can not be converted to a number\n',token)
  error('Double check input file');
end

