function K = stiffness(XY,E,M)
% stiffness  Compute stiffness matrix for 2D truss element
%
% Synopsis:  K = stiffness(XY,E,M)
%
% Input:     XY = nn by 2 matrix of global node positions, where nn is the
%                 number of nodes.  XY(i,1) is the x position of node i
%                 XY(i,2) is the y position.
%            E  = ne by 3 matrix of element definitions, where ne is the
%                 number of elements.  E(i,1) and E(i,2) are the global
%                 node numbers that define element i.  E(i,3) is an integer
%                 the type of material -- see M matrix, below
%            M  = nm by 4 matrix of material definitions, where nm is the
%                 total number of materials.  M(i,1) is an integer material
%                 ID number.  Usually, M(i,1) = i, but this is not necessary.
%                 The ID number is used in the element definition matrix,
%                 see E(i,3), above.  M(i,2) is the Young's modulus for the
%                 material.  M(i,3) is the cross-sectional area of the truss
%                 element.  M(i,4) is the density (mass/vol) of the material.
%
% Output:    K  = global, unconstrained stiffness matrix for 2D truss elements

% Copyright (c), 1998, Gerald Recktenwald, All rights reserved
% This m-file may be freely distributed, but it may not be sold.
% Version 1.01  7 July 1998,  gerry@me.pdx.edu

debug = 0;
[nn,ndim] = size(XY);        %  nn = # of nodes;  ndim = # of spatial dimensions
x = XY(:,1);   y = XY(:,2);  %  Make copies to make code easy to read

dof_per_node = 2;
aveConnectivity = 8;         %  Ave number of nodes connected to each node
                             %  determines memory allocation for stiffness matrix
nk = aveConnectivity*nn*dof_per_node;
K = spalloc(nn*dof_per_node,nn*dof_per_node,nk);

[ne,ncol] = size(E);
i1 = E(:,1);  i2 = E(:,2);
len = sqrt( (x(i2)-x(i1)).^2 + (y(i2)-y(i1)).^2);   %  length of each element
c = (x(i2)-x(i1))./len;          %  cosine of each element
s = (y(i2)-y(i1))./len;          %  sine of each element
mi = E(:,3);                     %  index of material for each element
eal = M(mi,2).*M(mi,3)./len;     %  product of E*area/length for each element
c2 = eal.*c.^2;                  %  Precompute c2, cs and s2 for efficiency
cs = eal.*c.*s;
s2 = eal.*s.^2;

for n=1:ne
  S = [ c2(n)   cs(n)   -c2(n)  -cs(n);
        cs(n)   s2(n)   -cs(n)  -s2(n);
       -c2(n)  -cs(n)    c2(n)   cs(n);
       -cs(n)  -s2(n)    cs(n)   s2(n)];

  % --- Add element stiffness to global stiffness
  gdof11 = 1 + 2*(i1(n)-1);   %  global DOF for first DOF of first node in element
  gdof22 = 1 + 2*(i2(n)-1);   %    "   "  "  "  first "   "  second node   "  " 
  K(gdof11:gdof11+1,gdof11:gdof11+1) = K(gdof11:gdof11+1,gdof11:gdof11+1) + S(1:2,1:2);
  K(gdof11:gdof11+1,gdof22:gdof22+1) = K(gdof11:gdof11+1,gdof22:gdof22+1) + S(1:2,3:4);
  K(gdof22:gdof22+1,gdof22:gdof22+1) = K(gdof22:gdof22+1,gdof22:gdof22+1) + S(3:4,3:4);
  K(gdof22:gdof22+1,gdof11:gdof11+1) = K(gdof22:gdof22+1,gdof11:gdof11+1) + S(3:4,1:2);

end

% --- Print info on sparsity of stiffness matrix
fprintf('FE mesh has %d nodes and %d elements.',nn,ne);
fprintf('     Sparsity of K = %g\n',nnz(K)/prod(size(K)));
if debug
  fprintf('%d matrix elements allocated for K\n',nk);
  fprintf('%d matrix elements in K are unused\n',nk - nnz(K));
end
