function [s,dl,dlr] = stress(XY,XYd,E,M)
% stress  Compute stress in 2D truss elements
%
% Synopsis:  s = stress(XY,XYd,E,M)
%
% Input:     XY  = nn by 2 matrix of global node positions, where nn is the
%                  number of nodes.  XY(i,1) is the x position of node i
%                  XY(i,2) is the y position.
%            XYd = same as XY except these are coordinates of the displaced nodes
%            E   = ne by 3 matrix of element definitions, where ne is the
%                  number of elements.  E(i,1) and E(i,2) are the global
%                  node numbers that define element i.  E(i,3) is an integer
%                  the type of material -- see M matrix, below
%            M   = nm by 4 matrix of material definitions, where nm is the
%                  total number of materials.  M(i,1) is an integer material
%                  ID number.  Usually, M(i,1) = i, but this is not necessary.
%                  The ID number is used in the element definition matrix,
%                  see E(i,3), above.  M(i,2) is the Young's modulus for the
%                  material.  M(i,3) is the cross-sectional area of the truss
%                  element.  M(i,4) is the density (mass/vol) of the material.
%
% Output:    s   = stress in each truss elements
%            dl  = (optional) absolute change in length of each element
%            dlr = (optional) relative change in length of each element

% Copyright (c), 1998, Gerald Recktenwald, All rights reserved
% This m-file may be freely distributed, but it may not be sold.
% Version 1.01  7 July 1998,  gerry@me.pdx.edu

[ne,ncol] = size(E);
i1 = E(:,1);  i2 = E(:,2);    %  global node numbers
emod = M( E(:,3),2 );         %  Young's modulus for each element
% --- len = original length, lend = displaced length,
%     dlen = change in length for each element
len  = sqrt( (XY(i2,1) -XY(i1,1) ).^2 + (XY(i2,2) -XY(i1,2) ).^2);
lend = sqrt( (XYd(i2,1)-XYd(i1,1)).^2 + (XYd(i2,2)-XYd(i1,2)).^2);
dlen = lend - len;
s = emod.*dlen./len;

if nargout>=2,  dl = dlen;      end
if nargout>=3,  dlr = dl./len;  end
